=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2007-02-12 05:09:17 +0000
+++ bzrlib/dirstate.py	2007-02-12 09:02:46 +0000
@@ -116,11 +116,10 @@
 import struct
 import zlib
 
+from bzrlib import errors
 import bzrlib.inventory
 from bzrlib.osutils import pathjoin, sha_file, sha_string, walkdirs
 
-# TODO:
-# 1)
 
 class DirState(object):
     """Record directory and metadata state for fast access.
@@ -414,7 +413,16 @@
         # FIXME: This is probably expensive - we encode the path that in the
         # common case will be the same across all parents, twice. 
         # also, id2path is slow in inventory, and we should make that fast.
-        parent_entry = parent_tree.inventory[fileid]
+        try:
+            parent_entry = parent_tree.inventory[fileid]
+        except errors.NoSuchId:
+            # this parent does not have fileid - return a bogus entry, which 
+            # will be filtered out on iteration etc.
+            # an empty revision id is bogus and safe to put on disk
+            # we make it be a 'file', just because. We give it the 
+            # deleted file path dirname and basename, 0 size, not executable
+            # and no sha1.
+            return ('', 'file', '/', 'RECYCLED.BIN', 0, False, '')
         parent_path = parent_tree.inventory.id2path(fileid)
         dirname, basename = os.path.split(parent_path.encode('utf8'))
         return (parent_entry.revision.encode('utf8'),

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2007-02-12 05:09:17 +0000
+++ bzrlib/tests/test_dirstate.py	2007-02-12 09:02:46 +0000
@@ -25,6 +25,8 @@
 
 
 # TODO:
+# test DirStateRevisionTree : test filtering out of deleted files does not
+#         filter out files called RECYCLED.BIN ;)
 # test 0 parents, 1 parent, 4 parents.
 # test unicode parents, non unicode parents
 # test all change permutations in one and two parents.
@@ -228,6 +230,7 @@
         # doing a save should work here as there have been no changes.
         state.save()
 
+
 class TestDirStateInitialize(TestCaseWithTransport):
 
     def test_initialize(self):

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2007-02-08 20:47:07 +0000
+++ bzrlib/tests/test_workingtree.py	2007-02-12 09:02:46 +0000
@@ -331,7 +331,6 @@
         # set the parents as if a merge had taken place. 
         # this should cause the tree data to be folded into the 
         # dirstate.
-        ## import pdb;pdb.set_trace()
         tree.set_parent_trees([
             (rev1, rev1_tree),
             (rev2, rev2_tree),

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2007-02-02 04:49:38 +0000
+++ bzrlib/tree.py	2007-02-12 09:02:46 +0000
@@ -166,7 +166,7 @@
         """
         raise NotImplementedError(self._comparison_data)
 
-    def _file_size(entry, stat_value):
+    def _file_size(self, entry, stat_value):
         raise NotImplementedError(self._file_size)
 
     def _get_inventory(self):

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2007-02-08 20:47:07 +0000
+++ bzrlib/workingtree_4.py	2007-02-12 09:02:46 +0000
@@ -61,7 +61,7 @@
 
 from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.inventory import InventoryEntry, Inventory, ROOT_ID
+from bzrlib.inventory import InventoryEntry, Inventory, ROOT_ID, make_entry
 from bzrlib.lockable_files import LockableFiles, TransportLock
 from bzrlib.lockdir import LockDir
 import bzrlib.mutabletree
@@ -360,6 +360,20 @@
         # sensible: the entry might not have come from us?
         return entry.kind, entry.executable, None
 
+    def _file_size(self, entry, stat_value):
+        return entry.text_size
+
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        # TODO: if path is present, fast-path on that, as inventory
+        # might not be present
+        ie = self.inventory[file_id]
+        if ie.kind == "file":
+            return ie.text_sha1
+        return None
+
+    def get_file_size(self, file_id):
+        return self.inventory[file_id].text_size
+
     def _get_inventory(self):
         if self._inventory is not None:
             return self._inventory
@@ -379,9 +393,25 @@
         assert self._revision_id in self._dirstate.get_parent_ids(), \
             'parent %s has disappeared from %s' % (
             self._revision_id, self._dirstate.get_parent_ids())
-        inv = Inventory()
-        for line in self._dirstate._iter_rows():
-            print line
+        parent_index = self._dirstate.get_parent_ids().index(self._revision_id)
+        rows = self._dirstate._iter_rows()
+        root_row = rows.next()
+        inv = Inventory(root_id=root_row[0][3].decode('utf8'),
+            revision_id=self._revision_id)
+        for line in rows:
+            revid, kind, dirname, name, size, executable, sha1 = line[1][parent_index]
+            if not revid:
+                # not in this revision tree.
+                continue
+            parent_id = inv[inv.path2id(dirname.decode('utf8'))].file_id
+            file_id = line[0][3].decode('utf8')
+            entry = make_entry(kind, name.decode('utf8'), parent_id, file_id)
+            entry.revision = revid.decode('utf8')
+            if kind == 'file':
+                entry.executable = executable
+                entry.text_size = size
+                entry.text_sha1 = sha1
+            inv.add(entry)
         self._inventory = inv
 
     def get_parent_ids(self):

